module sha1reqb(clk, rst, u_vld, u_idx, u_val, match, i_re, i_ra, i_rv, i_rd, i_irq);
input clk;
input rst;
input u_vld;
input [5:0] u_idx;
input [5:0] u_val;
input match;
input i_re;
input [2:0] i_ra;
output i_rv;
output [17:0] i_rd;
output i_irq;

reg [5:0] u_idx_q;
reg u_vld_q;
wire u_vld0 = u_vld && (u_idx == 6'b0);
wire u_vldx = u_vld && (u_idx[5:3] == 3'b0) && (u_idx[2:0] != 3'b0);
wire u_vld0_q = u_vld_q && (u_idx_q == 6'b0);
wire u_vldx_q = u_vld_q && (u_idx_q[5:3] == 3'b0) && (u_idx_q[2:0] != 3'b0);

wire [2:0] ram_ra = u_vldx ? u_idx : i_ra;
reg [2:0] ram_ra_q = 3'b000;
wire [17:0] ram_rd;
wire [17:0] ram_wd;

reg [5:0] u_val_q;
reg match_q;

reg [7:0] latch_map = 8'b0;
reg [7:0] latch2_map = 8'b0;
wire latch_map_i = latch_map[ram_ra_q];
wire latch2_map_i = latch2_map[ram_ra_q];

assign ram_wd[5:0] = u_val_q;
assign ram_wd[11:6] = latch_map_i ? ram_rd[11:6] : ram_rd[5:0];
assign ram_wd[17:12] = latch2_map_i ? ram_rd[17:12] : ram_rd[5:0];
wire [17:0] i_rd_x;
assign i_rd_x[5:0] = ram_rd[5:0];
assign i_rd_x[11:6] = latch_map_i ? ram_rd[11:6] : ram_rd[5:0];
assign i_rd_x[17:12] = latch2_map_i ? ram_rd[17:12] : ram_rd[5:0];

reg [5:0] r0_curr, r0_first, r0_last;
wire [17:0] i_rd_0_e = {r0_last, r0_first, r0_curr};
reg [17:0] i_rd_0;

reg latched = 1'b0;
reg clean_read = 1'b1;

wire [17:0] i_rd = {ram_ra_q ? i_rd_x : i_rd_0};
wire i_irq = !clean_read;
wire i_rv_e = i_re && !u_vldx;
reg i_rv;

wire [17:0] ram_wd_r = rst ? 18'h00000 : ram_wd;

always @(posedge clk) begin
	if(rst)
		latched <= 1'b0;
	else if(match_q)
		latched <= 1'b1;
	else if(clean_read && i_rv_e && ram_ra == 3'd7)
		latched <= 1'b0;
	u_vld_q <= u_vld;
 	u_idx_q <= u_idx;
	u_val_q <= u_val;
	match_q <= match;

	if(u_vld0_q) begin
		r0_curr <= u_val_q;
		if(match_q && !latched)
			r0_first <= u_val_q;
		if(match_q)
			r0_last <= u_val_q;
	end else begin
		if(match_q && !latched)
			r0_first <= r0_curr;
		if(match_q)
			r0_last <= r0_curr;
	end

	if((match_q && !latched) || rst)
		latch_map <= 8'b0;
	else if(latched && u_vldx_q)
		latch_map[ram_ra_q] <= 1'b1;
	if(match_q || rst)
		latch2_map <= 8'b0;
	else if(u_vldx_q)
		latch2_map[ram_ra_q] <= 1'b1;

	if(rst)
		ram_ra_q <= ram_ra_q + 1;
	else
		ram_ra_q <= ram_ra;
	i_rv <= i_rv_e;
	i_rd_0 <= i_rd_0_e;

	if(rst)
		clean_read <= 1'b1;
	else if(match_q)
		clean_read <= 1'b0;
	else if(i_rv_e && ram_ra == 3'd0)
		clean_read <= 1'b1;
end

wire ram_we = u_vldx_q || rst;
RAMB16_S18_S18 ram (
					.DOB(ram_rd[15:0]),
					.DOPB(ram_rd[17:16]),
	.ADDRA({7'h00, ram_ra_q}),	.ADDRB({7'h00, ram_ra}),
	.CLKA(clk),			.CLKB(clk),
	.DIA(ram_wd_r[15:0]),		.DIB(16'h0000),
	.DIPA(ram_wd_r[17:16]),		.DIPB(2'h0),
	.ENA(1'b1),			.ENB(1'b1),
	.SSRA(1'b0),			.SSRB(1'b0),
	.WEA(ram_we),			.WEB(1'b0)
);


endmodule
