module sha1(clk, r_we, r_re, r_ca, r_a, r_d, r_dv, irq, dcm_status);
input clk;
input r_we;
input r_re;
input [2:0] r_ca;
input [12:0] r_a;
inout [17:0] r_d;
output r_dv;
output irq;
input [7:0] dcm_status;

wire rst;
wire run;
wire [5:0] base;
wire wen;
wire ren;
wire [12:0] adr;
wire [17:0] wval;
wire [17:0] rval;
wire rdv;

wire i_p1t, i_p2t;
wire [8:0] i_p1b, i_p2b;
wire o_p1t, o_p2t;
wire [8:0] o_p1b, o_p2b;

wire match;
sha1rbif rbif (clk,
               r_we, r_re, r_ca, r_a, r_d, r_dv,
               rst, run, wen, ren, adr, wval, rval, rdv,
               i_p1t, i_p1b, i_p2t, i_p2b,
               o_p1t, o_p1b, o_p2t, o_p2b,
	       base,
               {6'b0, dcm_status[7:4], run, rst, irq, match, dcm_status[3:0]});

wire walk_rst = !run;
wire oen;
wire [5:0] oidx, oval;
sha1walk walk (clk, walk_rst, base, i_p1t, i_p1b, i_p2t, i_p2b, oen, oidx, oval);

wire [31:0] hash_out;
sha1hash hash (clk, rst, o_p1t, o_p1b, o_p2t, o_p2b, hash_out);

sha1comp comp (clk, 1'b1, hash_out, match, wen, adr, wval);

sha1reqb reqb (clk, walk_rst, oen, oidx, oval, match, ren, adr[2:0], rdv, rval, irq);

endmodule
