`define CHIP_ID 3'd4

module sha1rbif(clk,
                r_we, r_re, r_ca, r_a, r_d, r_dv,
		rst, run, wen, ren, adr, wval, rval, rdv,
		i_p1t, i_p1b, i_p2t, i_p2b,
		o_p1t, o_p1b, o_p2t, o_p2b,
		base, status);
input clk;
input r_we;
input r_re;
input [2:0] r_ca;
input [12:0] r_a;
inout [17:0] r_d;
output r_dv;

output rst;
output run;
output wen;
output ren;
output [12:0] adr;
output [17:0] wval;
input [17:0] rval;
input rdv;

input i_p1t, i_p2t;
input [8:0] i_p1b, i_p2b;
output o_p1t, o_p2t;
output [8:0] o_p1b, o_p2b;

output [5:0] base;
input [17:0] status;

reg [17:0] r_d_q;
reg [12:0] r_a_q;
reg [2:0] r_ca_q;

reg r_we_q, r_we_qq;
reg r_re_q, r_re_qq;
reg r_dv_int;
reg [17:0] rlatch;
reg ren_stat_q;
wire rdv_int = rdv || ren_stat_q;
wire wen = r_we_q && !r_we_qq && (r_ca_q == `CHIP_ID);
wire ren_int = r_re_q && r_re_qq && !r_dv_int && !rdv_int && (r_ca_q == `CHIP_ID);
wire ren_stat = ren_int && (r_a_q[3] == 1'b1);
wire ren = ren_int && (r_a_q[3] == 1'b0);

wire r_dv = r_dv_int ? 1'b1 : 1'bZ;

wire [17:0] r_d = r_dv_int ? rlatch : {18{1'bZ}};

always @(posedge clk) begin
	r_ca_q <= r_ca;
	r_a_q <= r_a;
	r_d_q <= r_d;
	r_we_q <= r_we;
	r_we_qq <= r_we_q;
	r_re_q <= r_re;
	r_re_qq <= r_re_q;
	if(rdv_int) begin
		rlatch <= ren_stat_q ? status : rval;
		r_dv_int <= 1'b1;
	end else if(!r_re_q)
		r_dv_int <= 1'b0;
	ren_stat_q <= ren_stat;
end

wire [12:0] adr = r_a_q;
wire [17:0] wval = r_d_q;

wire wr_ctrl = wen && (r_a_q[12:10] == 3'b111);

reg rst = 1'b1, run = 1'b0;
reg [5:0] base = 6'h07;
always @(posedge clk) begin
	if(wr_ctrl && r_a_q[1:0] == 2'b11) begin
		base <= r_d_q[11:6];
		rst <= r_d_q[0];
		run <= r_d_q[1];
	end
end

wire wr_init = wr_ctrl && (r_a_q[1] == 1'b0);
wire o_p1t = wr_init ? 1'b1 : i_p1t;
wire [8:0] o_p1b = wr_init ? r_d_q[8:0] : i_p1b;
wire o_p2t = wr_init ? r_a_q[0] : i_p2t;
wire [8:0] o_p2b = wr_init ? r_d_q[17:9] : i_p2b;

endmodule
