// dir: 0 = ++, 1 = --
module sha1cntr(clk, rst, oen, oidx, oval, odir);
input clk;
input rst;
output oen;
output [5:0] oidx;
output [5:0] oval;
output odir;

reg rstq;
reg [5:0] cntr0, cntr0q;
reg dir0;
wire [6:0] ramval;
wire [5:0] cntrval = ramval[5:0];
wire [5:0] cntrvalp = cntrval + 1;
wire cntrdir = ramval[6];
wire cntrdirp = cntrdir ^^ (cntrval==6'h3F);
wire [6:0] ramvalp = rstq ? 7'b0 : {cntrdirp, cntrvalp};
reg incr;
reg [5:0] active;
reg [5:0] actval;
reg actdir;
reg [1:0] encnt;
reg oen;
reg [5:0] oidx;
reg odir;
reg [5:0] oval;
always @(posedge clk) begin
    rstq <= rst;
    if(rst || rstq) begin
	if(rst != rstq)
	    cntr0q <= 6'h3F;
	else
	    cntr0q <= cntr0q + 1;
	cntr0 <= 6'h00;
	dir0 <= 1'b1;
	active <= 6'b0;
	actdir <= 1'b0;
	encnt <= 2'd0;
	oen <= 1'b0;
    end else begin
	cntr0 <= cntr0 + 1;
	cntr0q <= cntr0;
	incr <= (cntr0q==6'h3F) ? 1'b1 : (incr && cntrval==6'h3F);
	if(incr) begin
	    active <= cntr0q;
	    actdir <= cntrdirp;
	    actval <= cntrvalp;
	end
	if(encnt != 2'd2)
	    encnt <= encnt+1;
	else
	    oen <= 1'b1;
    end
    if(cntr0q==6'h00) begin
	oidx <= active + 1;
	odir <= actdir;
	oval <= actval ^ {6{actdir}};
	dir0 <= !dir0;
    end else begin
	oidx <= 6'h00;
	odir <= dir0;
	oval <= cntr0q ^ {6{dir0}};
    end
end

wire ram_we = incr || rstq;
wire ram_rd_hb;
RAMB16_S9_S9 ram (
					.DOB({ram_rd_hb, ramval}),
	.ADDRA({5'h00, cntr0q}),	.ADDRB({5'h00, cntr0}),
	.CLKA(clk),			.CLKB(clk),
	.DIA({1'b0, ramvalp}),		.DIB(8'h00),
	.DIPA(1'b0),			.DIPB(1'b0),
	.ENA(1'b1),			.ENB(1'b1),
	.SSRA(1'b0),			.SSRB(1'b0),
	.WEA(ram_we),			.WEB(1'b0)
);

endmodule
